/* This file was generated by syncqt. */
#ifndef QT_QTQUICKSHAPESDESIGNHELPERS_VERSION_H
#define QT_QTQUICKSHAPESDESIGNHELPERS_VERSION_H

#define QTQUICKSHAPESDESIGNHELPERS_VERSION_STR "6.11.1"

#define QTQUICKSHAPESDESIGNHELPERS_VERSION 0x0601101

#endif // QT_QTQUICKSHAPESDESIGNHELPERS_VERSION_H
