/* This file was generated by syncqt. */
#ifndef QT_QTQUICKVECTORIMAGEGENERATOR_VERSION_H
#define QT_QTQUICKVECTORIMAGEGENERATOR_VERSION_H

#define QTQUICKVECTORIMAGEGENERATOR_VERSION_STR "6.11.1"

#define QTQUICKVECTORIMAGEGENERATOR_VERSION 0x0601101

#endif // QT_QTQUICKVECTORIMAGEGENERATOR_VERSION_H
