// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:significant reason:default

#ifndef QQUICKOPENGLUTILS_H
#define QQUICKOPENGLUTILS_H

#if 0
#pragma qt_class(QQuickOpenGLUtils)
#endif

#include <QtQuick/qtquickglobal.h>

QT_BEGIN_NAMESPACE

namespace QQuickOpenGLUtils {

Q_QUICK_EXPORT void resetOpenGLState();

} // namespace QQuickOpenGLUtils

QT_END_NAMESPACE

#endif // QQUICKOPENGLUTILS_H
