/* This file was generated by syncqt. */
#ifndef QT_QTQMLTOOLINGSETTINGS_VERSION_H
#define QT_QTQMLTOOLINGSETTINGS_VERSION_H

#define QTQMLTOOLINGSETTINGS_VERSION_STR "6.11.1"

#define QTQMLTOOLINGSETTINGS_VERSION 0x0601101

#endif // QT_QTQMLTOOLINGSETTINGS_VERSION_H
