/* This file was generated by syncqt. */
#ifndef QT_QTQUICKCONTROLS2UNIVERSALSTYLEIMPL_VERSION_H
#define QT_QTQUICKCONTROLS2UNIVERSALSTYLEIMPL_VERSION_H

#define QTQUICKCONTROLS2UNIVERSALSTYLEIMPL_VERSION_STR "6.11.1"

#define QTQUICKCONTROLS2UNIVERSALSTYLEIMPL_VERSION 0x0601101

#endif // QT_QTQUICKCONTROLS2UNIVERSALSTYLEIMPL_VERSION_H
