// Copyright (C) 2017 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:significant reason:default

#ifndef QTQUICKCONTROLS2GLOBAL_H
#define QTQUICKCONTROLS2GLOBAL_H

#include <QtCore/qglobal.h>
#include <QtQuickControls2/qtquickcontrols2exports.h>

QT_BEGIN_NAMESPACE

Q_QUICKCONTROLS2_EXPORT void qml_register_types_QtQuick_Controls();

QT_END_NAMESPACE

#endif // QTQUICKCONTROLS2GLOBAL_H
